////////////////////////////////////////////////////////////////////////////////
//
//  M4Layer.fx
//  作成: ミーフォ茜
//
////////////////////////////////////////////////////////////////////////////////

// レイヤー名
#define LAYER_NAME Layer

// レンダリングターゲットを使用するか
// 0 にすると描画結果をそのまま使用するようになりますが、
// 他のレンダリングターゲットに対して使用できるようになります。
#define LAYER_RT 1

#define ALPHA_ENABLED 1

// レイヤーモード
// 0: 通常
// 1: 加算
// 2: 減算
// 3: 乗算
// 4: スクリーン
// 5: オーバーレイ
// 6: ハードライト
// 7: ソフトライト
// 8: ビビッドライト
// 9: リニアライト
// 10: ピンライト
// 11: 覆い焼き
// 12: 焼き込み
// 13: 比較 (暗)
// 14: 比較 (明)
// 15: 差の絶対値
// 16: 除外
// 17: 色相
// 18: 彩度
// 19: カラー
// 20: 輝度
// 21: remove identical pixels
// 22: replace removed pixels by new ones

float Tr : CONTROLOBJECT < string name = "(self)"; string item = "Tr"; >;
float Si : CONTROLOBJECT < string name = "(self)"; string item = "Si"; >;
float LayerMode : CONTROLOBJECT < string name = "(self)"; string item = "X"; >; //get the X field value to set the layer mode
float SwitchLayers : CONTROLOBJECT < string name = "(self)"; string item = "Rx"; >; //get the Rx field value to set the operation order
float ClipBelow : CONTROLOBJECT < string name = "(self)"; string item = "Y"; >; //get the Y field value to clip below layer input to [0;1]
float ClipAbove : CONTROLOBJECT < string name = "(self)"; string item = "Ry"; >; //get the Ry field value to clip above layer input to [0;1]
float LayerMask : CONTROLOBJECT < string name = "(self)"; string item = "Z"; >; //マスクを使用するか
float LayerMaskInvert : CONTROLOBJECT < string name = "(self)"; string item = "Rz"; >; //マスクを反転するか

////////////////////////////////////////////////////////////////

#include "..\Shared\M4LayerCore.fxsub"

// ポストエフェクト宣言
float Script : STANDARDSGLOBAL
<
    string ScriptOutput = "color";
    string ScriptClass = "scene";
    string ScriptOrder = "postprocess";
> = 0.8;

// スクリーンサイズ
float2 ViewportSize : VIEWPORTPIXELSIZE;
static const float2 ViewportOffset = float2(0.5, 0.5) / ViewportSize;

// 作業用テクスチャ
texture2D DepthBuffer : RENDERDEPTHSTENCILTARGET
<
	float2 ViewPortRatio = { 1.0, 1.0 };
	string Format        = "D24S8";
>;
texture2D ScreenBuffer : RENDERCOLORTARGET
<
	float2 ViewPortRatio = { 1.0, 1.0 };
	int    MipLevels     = 1;
	string Format = "D3DFMT_A16B16G16R16F";
>;
sampler2D ScreenSampler = sampler_state
{
	texture   = <ScreenBuffer>;
	MinFilter = NONE;
	MagFilter = NONE;
	MipFilter = NONE;
	AddressU  = CLAMP;
	AddressV  = CLAMP;
};

#if LAYER_RT
texture MERGE(LAYER_NAME, RT) : OFFSCREENRENDERTARGET
<
	string Description   = "Render Target for M4Layer.fx";
	float4 ClearColor    = { 0, 0, 0, 0 };
	float  ClearDepth    = 1.0;
	string Format = "D3DFMT_A16B16G16R16F";
	bool   AntiAlias     = true;
	int    Miplevels     = 1;
	string DefaultEffect = "self = hide; * = none;";
>;
sampler LayerSampler = sampler_state
{
	texture   = <MERGE(LAYER_NAME, RT)>;
	MinFilter = POINT;
	MagFilter = POINT;
	AddressU  = CLAMP;
	AddressV  = CLAMP;
};

#endif

texture MERGE(LAYER_NAME, MaskRT) : OFFSCREENRENDERTARGET
<
	string Description   = "Masking Render Target for M4Layer.fx";
	float4 ClearColor    = { 0, 0, 0, 0 };
	float  ClearDepth    = 1.0;
	string Format = "D3DFMT_A16B16G16R16F";
	bool   AntiAlias     = true;
	int    Miplevels     = 1;
	string DefaultEffect = "self = hide; * = VisibleMask.fx;";
>;
sampler LayerMaskSampler = sampler_state
{
	texture   = <MERGE(LAYER_NAME, MaskRT)>;
	MinFilter = POINT;
	MagFilter = POINT;
	AddressU  = CLAMP;
	AddressV  = CLAMP;
};

struct VS_OUTPUT
{
	float4 Pos : POSITION;
	float2 Tex : TEXCOORD0;
};

VS_OUTPUT Blend_VS(float4 Pos : POSITION, float2 Tex : TEXCOORD0)
{ 
	VS_OUTPUT Out;
	
	Out.Pos = Pos;
	Out.Tex = Tex + ViewportOffset;
	
	return Out;
}

float4 Blend_PS(float2 Tex: TEXCOORD0) : COLOR
{
	float4 background = tex2D(ScreenSampler, Tex);
	
#if LAYER_RT
	float4 foreground = tex2D(LayerSampler, Tex);
#else
	float4 foreground = background;
#endif
	
if (LayerMask)
{
	float4 m = tex2D(LayerMaskSampler, Tex);
	
	if (LayerMaskInvert)
	{
	foreground.a *= m.r * m.a;
	}else{
	foreground.a *= 1 - m.r * m.a;
	}
}
	
	float opacity = Tr * (Si / 10) * foreground.a;
	
	SwitchLayers *= 180/3.14159265;
	ClipAbove *= 180/3.14159265;
	
	background = Blend4(background, foreground, opacity);

	return background;
}

////////////////////////////////////////////////////////////////
// エフェクトテクニック
//
float4 ClearColor = { 0, 0, 0, 0 };
float  ClearDepth = 1;

technique PostEffectTec
<
	string Script =
		"RenderColorTarget0=ScreenBuffer;"
		"RenderDepthStencilTarget=DepthBuffer;"
#if ALPHA_ENABLED
		"ClearSetColor=ClearColor;"
		"ClearSetDepth=ClearDepth;"
#endif
		"Clear=Color;"
		"Clear=Depth;"
		"ScriptExternal=Color;"
		"RenderColorTarget0=;"
		"RenderDepthStencilTarget=;"
#if ALPHA_ENABLED
		"ClearSetColor=ClearColor;"
		"ClearSetDepth=ClearDepth;"
#endif
		"Clear=Color;"
		"Clear=Depth;"
		"Pass=PassBlend;";
>
{
	pass PassBlend < string Script = "Draw=Buffer;"; >
	{
		AlphaBlendEnable = true;
		VertexShader = compile vs_3_0 Blend_VS();
		PixelShader = compile ps_3_0 Blend_PS();
	}
};
