#include "..\Shared\M4ShaderCompat.fxsub"

float4x4 WorldViewProjMatrix : WORLDVIEWPROJECTION;
float4   MaterialDiffuse     : DIFFUSE < string Object = "Geometry"; >;

texture ObjectTexture: MATERIALTEXTURE;
sampler ObjTexSampler = sampler_state
{
	texture   = <ObjectTexture>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};

struct VS_OUTPUT
{
	float4 Pos : POSITION;
	float2 Tex : TEXCOORD0;
};

VS_OUTPUT Basic_VS(MMM_SKINNING_INPUT IN)
{
	float4 Pos = Compat_GetSkinnedPosition(IN);
	float2 Tex = IN.Tex;
	VS_OUTPUT Out = (VS_OUTPUT)0;

	Out.Pos = mul(Pos, WorldViewProjMatrix);
	Out.Tex = Tex;

	return Out;
}

float4 Basic_PS(VS_OUTPUT IN, uniform bool useTexture) : COLOR0
{
	float alpha = MaterialDiffuse.a;

	if (useTexture) alpha *= tex2D(ObjTexSampler, IN.Tex).a;

	return float4(0, 0, 0, alpha);
}

technique NoSSNoTexture < string MMDPass = "object"; bool UseTexture = false; >
{
	pass P
	{ 
		VertexShader = compile vs_2_0 Basic_VS();
		PixelShader = compile ps_2_0 Basic_PS(false); 
	}
}

technique WithSSNoTexture < string MMDPass = "object_ss"; bool UseTexture = false; >
{
	pass P
	{ 
		VertexShader = compile vs_2_0 Basic_VS();
		PixelShader = compile ps_2_0 Basic_PS(false); 
	}
}

technique NoSSWithTexture < string MMDPass = "object"; bool UseTexture = true; >
{
	pass P
	{ 
		VertexShader = compile vs_2_0 Basic_VS();
		PixelShader = compile ps_2_0 Basic_PS(true); 
	}
}

technique WithSSWithTexture < string MMDPass = "object_ss"; bool UseTexture = true; >
{
	pass P
	{ 
		VertexShader = compile vs_2_0 Basic_VS();
		PixelShader = compile ps_2_0 Basic_PS(true); 
	}
}

technique EdgeTec < string MMDPass = "edge"; > { }
technique ShadowTec < string MMDPass = "shadow"; > { }
technique ZplotTec < string MMDPass = "zplot"; > { }
